-- StubbleCultivator
-- Spec for StubbleCultivator to be used with chopped straw
-- When activated, the cultivator will work the soil but let the choppedStraw layer to have distinct visible behavior between Stubble Cultivator and Standard Cultivator
-- by yumi

source(Utils.getFilename("StubbleCultivatorEvent.lua", g_currentModDirectory))

StubbleCultivator = {};

function StubbleCultivator.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(StubbleCultivator, specializations);
end;

function StubbleCultivator:load(xmlFile)
  self.setStubbleCultivatorActive = SpecializationUtil.callSpecializationsFunction("setStubbleCultivatorActive");
  self.isStubbleCultivatorActive = false;
end;

function StubbleCultivator:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
  if not resetVehicles then
    local isStubbleCultivatorActive= Utils.getNoNil(getXMLBool(xmlFile, key.."#StubbleCultivator"),false);

    if isStubbleCultivatorActive == nil then
      self:setStubbleCultivatorActive(false);
      --self.isStubbleCultivatorActive = false;
    else
      self:setStubbleCultivatorActive(isStubbleCultivatorActive);
      --self.isStubbleCultivatorActive = isStubbleCultivatorActive;
    end

  end;
  return BaseMission.VEHICLE_LOAD_OK;
end;
 
function StubbleCultivator:getSaveAttributesAndNodes(nodeIdent)
  local attributes = 'StubbleCultivator="'.. tostring(self.isStubbleCultivatorActive) .. '"';
  return attributes, nil;
end;

function StubbleCultivator:delete()
end;

function StubbleCultivator:readStream(streamId, connection)
  self:setStubbleCultivatorActive(streamReadBool(streamId), true);
end;

function StubbleCultivator:writeStream(streamId, connection)
  streamWriteBool(streamId, self.isStubbleCultivatorActive);
end;

function StubbleCultivator:mouseEvent(posX, posY, isDown, isUp, button)

end;

function StubbleCultivator:keyEvent(unicode, sym, modifier, isDown)
end;

function StubbleCultivator:onLeave()

end

function StubbleCultivator:update(dt)
  if self:getIsActiveForInput() and self.isClient then
    if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
      self:setStubbleCultivatorActive(not self.isStubbleCultivatorActive);
    end
  end
end;

function StubbleCultivator:updateTick(dt)

  local preparingOutputId = nil;
  
  if self.cultivatorHasGroundContact and self:isLowered() then
    if g_currentMission.cs_strawBindings ~= nil and self.isStubbleCultivatorActive then
      local fruitIdx = nil;
      
      local cx,cz;
      cx,_,cz = getWorldTranslation(self.rootNode);

      if cx ~= nil and cx==cx and cz==cz then -- Make extra sure that there actually is an x,z coordinate to use.
        local squareSize = 10
        local widthX,widthZ, heightX,heightZ = squareSize-0.5,0, 0,squareSize-0.5
        local x,z = cx - (squareSize/2), cz - (squareSize/2)
          
        -- Get current fruit on the groud
        local foundFruits = nil;
        for fruitIndex,fruit in pairs(g_currentMission.fruits) do
          if fruit.id ~= nil and fruit.id ~= 0 then

            setDensityCompareParams(fruit.id, "between", 2, 10)  -- #fruit from 1st growing state to withered state
            local _,numPixels,_ = getDensityParallelogram(fruit.id, x,z, widthX,widthZ, heightX,heightZ, 0,g_currentMission.numFruitStateChannels)
            setDensityCompareParams(fruit.id, "greater", 0)

            if numPixels > 0 then
              local fillTypeIndex = FruitUtil.fruitTypeToFillType[fruitIndex]
              local fillTypeName = FillUtil.fillTypeIndexToDesc[fillTypeIndex].nameI18N
              if fillTypeName == nil then
                fillTypeName = FillUtil.fillTypeIndexToDesc[fillTypeIndex].name
                if g_i18n:hasText(fillTypeName) then
                  fillTypeName = g_i18n:getText(fillTypeName)
                end
              end
              foundFruits = ((foundFruits == nil) and "" or foundFruits..", ") .. fillTypeName
              --print("We have found some " .. fillTypeName);
              --print("numPixels: " .. tostring(numPixels > 0))
              -- keep 1st one for now
              if fruitIdx == nil then
                fruitIdx = fruitIndex;
                break;
              end
            end
          end
        end
      end            
      
      if fruitIdx ~= nil then
        local fruitDesc = FruitUtil.fruitIndexToDesc[fruitIdx];
        local choppedStrawBinding = g_currentMission.cs_strawBindings[fruitDesc.name];
        if choppedStrawBinding ~= nil then
          local strawTypeFoliageId = g_currentMission.cs_strawTypes[choppedStrawBinding.strawTypeId].foliageId
          if strawTypeFoliageId ~= nil then
            for _,workArea in pairs(self.workAreas) do
              local x, _, z = getWorldTranslation(workArea.start)
              local x1, _, z1 = getWorldTranslation(workArea.width)
              local x2, _, z2 = getWorldTranslation(workArea.height)
              
              Utils.updateStrawArea(strawTypeFoliageId, x, z, x1, z1, x2, z2)
            end
          end
        end
      end;
    end
  end
end;

function StubbleCultivator:draw()
		--display a help message to say how to desactivate the lock
    if self.isStubbleCultivatorActive then
      g_currentMission:addHelpButtonText(StubbleCultivator.l10nTexts["StubbleCultivator_Deactivate"], InputBinding.IMPLEMENT_EXTRA); --g_i18n:getText("StubbleCultivator_Deactivate"), InputBinding.OnOff);
    else
      g_currentMission:addHelpButtonText(StubbleCultivator.l10nTexts["StubbleCultivator_Activate"], InputBinding.IMPLEMENT_EXTRA); --g_i18n:getText("StubbleCultivator_Activate"), InputBinding.OnOff);
    end
end;

function StubbleCultivator:attachImplement(implement)
end;

function StubbleCultivator:setStubbleCultivatorActive(isActive, noEventSend)
  self.isStubbleCultivatorActive = isActive;
  if noEventSend ~= true then
    StubbleCultivatorEvent.sendEvent(self, isActive);
  end;
end;
